-- Copyright (C) 2009-2010 John Millikin <jmillikin@gmail.com>
-- 
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.

{-# LANGUAGE OverloadedStrings #-}
module Main (main) where

import DBus.Client
import Control.Concurrent (forkIO)
import Control.Monad (forever)
import Data.List (sort)

dbus :: Proxy
dbus = Proxy "org.freedesktop.DBus" "/org/freedesktop/DBus" "org.freedesktop.DBus"

main :: IO ()
main = do
	-- Connect to the bus, and print which name was assigned to this
	-- connection.
	client <- newClient =<< getSessionBus
	putStrLn $ "Connected as: " ++ show (clientName client)
	
	-- Start message handling in another thread
	forkIO $ runDBus client mainLoop
	
	-- Request a list of connected clients from the bus
	names <- runDBus client $ do
		reply <- callProxyBlocking_ dbus "ListNames" [] []
		
		-- Pull out the body, and convert it to [String]
		let Just names = fromArray =<< fromVariant (messageBody reply !! 0)
		return names
	
	-- Print each name on a line, sorted so reserved names are below
	-- temporary names.
	mapM_ putStrLn $ sort names
